import sys
import QUB
import Buttons
import Dialogs
import os.path
import qubtree

from math import *

import win32com.client
from pythoncom import CoInitialize, CoUninitialize


def GenColumn(sheet, startRow, col):
    '''yields the values of cells in column col, starting with row startRow, until an empty row'''

    cells = sheet.Cells
    for row in xrange(startRow, 10000000):
        val = cells(row, col).Value
        if val == None:
            return
        yield val

def GenInts(gen):
    for x in gen:
        try:
            yield int(x)
        except:
            try:
                yield int(round(float(x)))
            except:
                return

def GenDwells(seq):
    count = 0
    cls = -1

    try:
        while True:
            next = seq.next()
            if next != cls:
                if count:
                    yield (cls, count)
                count = 0
                cls = next
            count = count + 1
    except StopIteration:
        pass
    
    if count:
        yield (cls, count)

# events = GenDwells( GenInts(GenColumn(sheet, 2, col)) )

class SelLists:
    def __init__(self):
        self.lists = {}
        
    def getList(self, name):
        try:
            return self.lists[name]
        except:
            self.lists[name] = []
            return self.lists[name]
        
    def add(self, name, selname, first, last):
        self.getList(name).append( [selname, first, last] )
        
    def toTree(self):
        tree = qubtree.Node('Lists')
        for listname in self.lists.keys():
            lst = tree.append('List')
            lst.data = listname
            
            sels = self.lists[listname]
            selsNode = lst['Sels']
            selsNode.data.setup(qubtree.QTR_TYPE_INT, len(sels), 2)

            i = 0
            for nm, first, last in sels:
                selsNode.data.row(i)[0] = first
                selsNode.data.row(i)[1] = last
                sel = lst.append(nm).data
                sel.setup(qubtree.QTR_TYPE_INT, 1, 2)
                sel[0] = first
                sel[1] = last
                i = i + 1
        return tree

def ImportSleepData(props):
    status = QUB.CreateWizard()
    try:
        status.caption = 'Importing sleep data...'
        statbase = ''
        status.message = statbase + '\r\nMouse: %s' % ''
        status.show()
        
        CoInitialize()

        fname = os.path.splitext(props['fname'])
        sampling = props['sampling']

        excel = win32com.client.Dispatch('Excel.Application')
        excel.Visible = True
        book = excel.Workbooks.Open(fname[0] + fname[1])

        for sheet in [book.Worksheets(i) for i in range(1, 1+book.Worksheets.Count)]:
            sheetName = sheet.Name
            statbase = statbase + sheetName + '...'
            qsf = qubtree.Node('SessionFile')
            dwt = open(fname[0] + '-' + sheetName + '.dwt', "w")
            segno = 1
            atPoint, atTime = 0, 0.0
            lists = SelLists()

            col = 1
            while col <= sheet.Columns.Count:
                if sheet.Cells(1, col).Value:
                    tag = str(sheet.Cells(1, col))
                    col = col + 1
                    while sheet.Cells(1, col).Value:
                        group = str(sheet.Cells(1, col))
                        status.message = statbase + '\r\nMouse: %s\r\n%s' % (tag, group)
                        first = atPoint
                        start = atTime
                        events = [x for x in GenDwells( GenInts( GenColumn(sheet, 2, col) ) )]
                        if len(events):
                            dwt.write('Segment: %d Dwells: %d Sampling(ms): %f Start(ms): %f ClassCount: 3 1 0 2 0 3 0\n' % (segno, len(events), sampling, start))
                            for cls, count in events:
                                dwt.write('\t%d\t%f\n' % (max(cls-1,0), count*sampling))
                                atPoint = atPoint + count
                                atTime = atTime + count*sampling
                            dwt.write('\n')
                            segno = segno + 1
                            lists.add(group, tag, first, atPoint-1)
                        col = col + 1
                col = col + 1

            dwt.close()
            qsf.append( lists.toTree() )
            qsf.saveAs(fname[0] + '-' + sheetName + '.qsf')
            del qsf

            statbase = statbase + 'Done.\r\n'

        status.message = statbase
        status.buttons = ['OK']
        status.waitForButton()

    finally:
        CoUninitialize()
        status.hide()

Buttons.AddTaskButton( 'Imprt Sleep', # button text
                       ImportSleepData,   # function
                       'ImpSlp',       # short text for small buttons
                       1,            # show settings dialog on left-click?
                       1,            # show settings dialog on right-click?
                       "Convert a sleep kinetics spreadsheet into QUB data",   # tool-tip
                       "Convert a sleep kinetics spreadsheet into QUB data",   # description in "Edit Buttons"
                       "Import Sleep Data:", # caption for the settings dialog
                       [Buttons.TextField("fname", ".xls file:", "", Dialogs.acceptString),
                        Buttons.TextField("sampling", "sampling(ms):", "15.0", Dialogs.acceptFloat)
                        ],
                       'Scripts' )